/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity;

import org.databene.commons.ArrayFormat;
import org.databene.commons.xml.SimpleXMLWriter;
import org.databene.jdbacl.model.DBCheckConstraint;
import org.databene.jdbacl.model.DBForeignKeyConstraint;
import org.databene.jdbacl.model.DBUniqueConstraint;
import org.xml.sax.helpers.AttributesImpl;
import static org.databene.commons.xml.SimpleXMLWriter.*;

/**
 * Provides methods for creating check type definitions in XML programmatically.<br/><br/>
 * Created: 26.05.2011 07:36:51
 * @since 0.8
 * @author Volker Bergmann
 */
public class CheckDefinitionUtil {

	private static final String NAME = "name";
	private static final String TABLE = "table";
	private static final String DEFECT_TYPE = "defectType";
	private static final String AUTHOR = "author";
	private static final String TAGS = "tags";

	private CheckDefinitionUtil() { }
	
	public static void defineUniqueKeyCheck(String checkName, DBUniqueConstraint uk, String author, String defectType, String tag, SimpleXMLWriter writer) {
		defineUniqueKeyCheck(checkName, uk.getTable().getName(), uk.getColumnNames(), author, defectType, tag, writer);
	}
	
	public static void defineUniqueKeyCheck(String checkName, String tableName, String[] columnNames, String author, String defectType, String tag, SimpleXMLWriter writer) {
		try {
			String csColNames = ArrayFormat.format(columnNames);

			// write check
			writer.startElement("check", 
				NAME, 		checkName.toString(),
				TABLE, 		tableName,
				DEFECT_TYPE, 	defectType,
				AUTHOR,		author,
				TAGS,		tag
				);
			writer.startElement("unique", 
				"columns", csColNames);
			writer.endElement("unique");
			writer.endElement("check");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void defineStringLengthCheck(String checkName, String tableName, String columnExpression, 
			Integer minLength, Integer maxLength, String author, String defectType, String tag, SimpleXMLWriter writer) {
		try {
			// write check
			writer.startElement("check", 
				NAME, 		checkName,
				TABLE, 		tableName,
				AUTHOR,		author,
				DEFECT_TYPE,defectType,
				TAGS,		tag
			);
			AttributesImpl atts = createAttributes("expression", columnExpression);
			if (minLength != null)
				addAttribute("maxLength", minLength.toString(), atts);
			if (maxLength != null)
				addAttribute("maxLength", maxLength.toString(), atts);
			writer.startElement("stringLength", atts);
			writer.endElement("stringLength");
			writer.endElement("check");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void defineNumberSizeCheck(String checkName, String tableName, String columnExpression, 
			Integer totalDigits, Integer fractionDigits, String author, String defectType, String tag, SimpleXMLWriter writer) {
		try {
			// write check
			writer.startElement("check", 
				NAME, 		checkName,
				TABLE, 		tableName,
				AUTHOR,		author,
				DEFECT_TYPE, defectType,
				TAGS,		tag);
			AttributesImpl atts = createAttributes("expression", columnExpression);
			if (totalDigits != null)
				addAttribute("totalDigits", totalDigits.toString(), atts);
			if (fractionDigits != null)
				addAttribute("fractionDigits" , fractionDigits.toString(), atts);
			writer.startElement("numberSize", atts);
			writer.endElement("numberSize");
			writer.endElement("check");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void defineForeignKeyCheck(String checkName, DBForeignKeyConstraint fk, String author, String defectType, String tag, SimpleXMLWriter out) {
		try {
			out.startElement("check", 
				NAME, 		checkName,
				TABLE, 		fk.getTable().getName(),
				DEFECT_TYPE,defectType,
				AUTHOR, 	author,
				TAGS,		tag
			);
			AttributesImpl fkAtts = createAttributes("refererColumns", ArrayFormat.format(fk.getColumnNames()));
			addAttribute("refereeTable", fk.getRefereeTable().getName(), fkAtts);
			addAttribute("refereeColumns", ArrayFormat.format(fk.getRefereeColumnNames()), fkAtts);
			out.startElement("foreignKey", fkAtts);
			out.endElement("foreignKey");
			out.endElement("check");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void defineNotNullCheck(String checkName, String tableName, String expression, String author, String defectType, String tag, SimpleXMLWriter writer) {
		try {
			writer.startElement("check", 
				NAME, 		checkName,
				TABLE, 		tableName,
				DEFECT_TYPE,defectType,
				AUTHOR,		author,
				TAGS,		tag);
			writer.startElement("notNull", 
				"expression", expression);
			writer.endElement("notNull");
			writer.endElement("check");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void defineCheckConstraintCheck(String checkName, DBCheckConstraint constraint, 
			String author, String defectType, String tag, SimpleXMLWriter writer) {
		try {
			String tableName = constraint.getTable().getName();
			writer.startElement("check", 
				NAME, 			checkName,
				TABLE, 			tableName,
				DEFECT_TYPE,	defectType,
				AUTHOR,			author,
				TAGS,		tag);
			writer.startElement("sql");
			writer.print("select * from " + tableName + " where not (" + constraint.getConditionText() + ")");
			writer.endElement("sql");
			writer.endElement("check");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
