/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Locale;
import java.util.Set;

import org.databene.commons.BeanUtil;
import org.databene.commons.ConfigurationError;
import org.databene.commons.ConnectFailedException;
import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.ImportFailedException;
import org.databene.commons.JDBCConnectData;
import org.databene.commons.StringUtil;
import org.databene.commons.SystemInfo;
import org.databene.commons.version.VersionNumber;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.TestSuiteLogger;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.parser.TestSuiteParser;
import org.databene.dbsanity.report.CheckAssessmentModule;
import org.databene.dbsanity.report.DefectsPerCheckModule;
import org.databene.dbsanity.report.DefectsPerTypeModule;
import org.databene.dbsanity.report.DefectsPerTableModule;
import org.databene.dbsanity.report.ErrorModule;
import org.databene.dbsanity.report.DefectsPageModule;
import org.databene.dbsanity.report.HTMLCheckExporter;
import org.databene.dbsanity.report.DigestModule;
import org.databene.dbsanity.report.InfoModule;
import org.databene.dbsanity.report.ReportContext;
import org.databene.dbsanity.report.ExecutionTimeModule;
import org.databene.dbsanity.report.UncheckedTablesModule;
import org.databene.dbsanity.report.VerdictSummaryModule;
import org.databene.dbsanity.report.SubSuiteModule;
import org.databene.dbsanity.report.TempResultFileModule;
import org.databene.html.HTMLUtil;
import org.databene.jdbacl.DBUtil;
import org.databene.jdbacl.identity.IdentityProvider;
import org.databene.jdbacl.identity.KeyMapper;
import org.databene.jdbacl.identity.mem.MemKeyMapper;
import org.databene.jdbacl.model.DBMetaDataImporter;
import org.databene.jdbacl.model.Database;
import org.databene.jdbacl.model.cache.CachingDBImporter;
import org.databene.jdbacl.model.jdbc.JDBCDBImporter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;

/**
 * Main class that reads the arguments, performs the checks and creates a
 * report. As command line argument, an environment name is expected (e.g.
 * 'mydb'), which is mapped to a properties file name (e.g.
 * 'mydb.env.properties') and the JDBC connection info is read from it. By
 * default, the test definitions are expected in the working directory's sub
 * folder 'test' and the report is written to the working directory's sub folder
 * 'result'.<br/>
 * <br/>
 * Created: 13.10.2010 13:40:56
 * 
 * @since 1.0
 * @author Volker Bergmann
 */
public class DbSanity {

	private static final Logger LOGGER = LoggerFactory.getLogger(DbSanity.class);
	
	static final String DEFAULT_CHECK_DEFINITION_FOLDER = "dbsanity";

	static final String DEFAULT_REPORT_FOLDER = "dbsanity-report";

	static final String DEFAULT_TEST_TEMP_RESULT_FOLDER = "temp";

	public static final String DEFAULT_STYLE = "online";

	// attributes --------------------------------------------------------------

	File checkDefinitionFile;
	File tempFolder;
	File reportFolder;
	private String style = DEFAULT_STYLE;
	boolean clearBefore;
	boolean metaDataCached;
	boolean deleteTempAfter;
	boolean condensed;
	private ExecutionMode mode;
	private Locale locale;
	private String ignorableTables;
	private String[] tables;
	
	SanityCheckSuite rootSuite;
	JDBCConnectData connectData;

	private String environment;

	private VersionNumber appVersion;

	private boolean browsing;

	private int defectCountLimit;

	private Set<String> tags;

	// constructors ------------------------------------------------------------

	public DbSanity() {
		this(new File(DEFAULT_CHECK_DEFINITION_FOLDER), null);
	}

	public DbSanity(File checkDefinitionFile, JDBCConnectData connectData) {
		this.checkDefinitionFile = checkDefinitionFile;
		this.tempFolder = new File(DEFAULT_TEST_TEMP_RESULT_FOLDER); // TODO v1.0 take the system's temp directory
		this.reportFolder = new File(DEFAULT_REPORT_FOLDER);
		this.connectData = connectData;
		this.browsing = false;
		this.clearBefore = false;
		this.metaDataCached = false;
		this.deleteTempAfter = true;
		this.condensed = false;
		this.mode = ExecutionMode.DEFAULT;
		this.locale = Locale.getDefault();
		this.defectCountLimit = 0;
		this.ignorableTables = null;
		this.tables = null;
	}

	// properties --------------------------------------------------------------

	public void setEnvironment(String environment) throws IOException {
		this.environment = environment;
		setConnectData(DBUtil.getConnectData(environment));
	}

	public JDBCConnectData getConnectData() {
		return connectData;
	}

	public void setConnectData(JDBCConnectData connectData) {
		this.connectData = connectData;
	}

	public void setAppVersion(VersionNumber appVersion) {
		this.appVersion = appVersion;
	}

	public File getCheckDefinitionFile() {
		return checkDefinitionFile;
	}

	public void setCheckDefinitionFile(File testDefinitionFolder) {
		this.checkDefinitionFile = testDefinitionFolder;
	}

	public void setTempFolder(File tempFolder) {
    	this.tempFolder = tempFolder;
    }

	public File getReportFolder() {
    	return reportFolder;
    }

	public void setReportFolder(File reportFolder) {
    	this.reportFolder = reportFolder;
    }

	public void setTables(String[] tables) {
		this.tables = tables;
	}

	public void setBrowsing(boolean browsing) {
		this.browsing = browsing;
	}

	public void setClearBefore(boolean clearBefore) {
    	this.clearBefore = clearBefore;
    }

	public void setMetaDataCached(boolean metaDataCached) {
		this.metaDataCached = metaDataCached;
	}
	
	public void setDeleteTempAfter(boolean deleteTempAfter) {
		this.deleteTempAfter = deleteTempAfter;
	}

	public void setCondensed(boolean condensed) {
		this.condensed = condensed;
	}
	
	public void setSkin(String style) {
	    this.style = style;
    }

	public ExecutionMode getMode() {
		return mode;
	}
	
	public void setMode(ExecutionMode mode) {
		this.mode = mode;
	}

	public Locale getLocale() {
		return locale;
	}
	
	public void setLocale(Locale locale) {
		this.locale = locale;
	}

	// interface ---------------------------------------------------------------

	public SanityCheckSuite execute() throws Exception {
		checkPreconditions();

		// clear temp and report folders if they exist and shall be cleared
		if (clearBefore) {
			FileUtil.deleteDirectoryIfExists(tempFolder);
			FileUtil.deleteDirectoryIfExists(reportFolder);
		}
		
		// create temp folder beside test definition folder
		File tempFolder = new File(checkDefinitionFile.getParent(), DEFAULT_TEST_TEMP_RESULT_FOLDER); // TODO v0.x use system temp folder
		FileUtil.ensureDirectoryExists(tempFolder);

		// find out ignorable tables
		findIgnorableTables(checkDefinitionFile, true);
		
		// read database meta data
		Connection connection = openConnection();		
		Database database = getMetaData();
		
		// perform checks
	    IdentityProvider identityProvider = new IdentityProvider();
		KeyMapper keyMapper = createKeyMapper(connection, database.getName(), identityProvider);
		rootSuite = new TestSuiteParser().parseHierarchy(
				checkDefinitionFile, reportFolder, reportFolder, tempFolder, database, keyMapper, identityProvider);
		
		// create report context 
	    ReportContext context = createContext();
	    context.set("executionMode", mode);
	    Reporter reporter = createReporter(context, database);

	    try {
		    // perform checks
			rootSuite.perform(connection, reporter, context);
			reportFolder.mkdir();
		} finally {
			// create summary...
			reporter.summary(rootSuite);
			// ...and close everything
			DBUtil.close(connection);
		    IOUtil.close(reporter);
		    if (deleteTempAfter)
		    	FileUtil.deleteDirectoryIfExists(tempFolder);
		}
		new TestSuiteLogger().log(rootSuite);

		// create HTML versions of the check definitions
		renderSuiteInHTML(rootSuite);
		
		LOGGER.info("Sanity check finished");
		String reportRoot = reportFolder.getCanonicalPath() + File.separator + "index.html";
		if (mode != ExecutionMode.QUIET)
			System.out.println("The DBSanity report is available at " + reportRoot);
		
		if (browsing)
			HTMLUtil.openBrowser("file://" + reportRoot);
		
		return rootSuite;
	}

	private void renderSuiteInHTML(SanityCheckSuite suite) throws IOException {
		if (suite instanceof SanityCheckFolder) {
			for (SanityCheckSuite child : ((SanityCheckFolder) suite).getChildSuites().values())
				renderSuiteInHTML(child);
		} else {
			SanityCheckFile file = (SanityCheckFile) suite;
			new HTMLCheckExporter().export(file, reportFolder, style);
		}
	}

	private void findIgnorableTables(File fileOrFolder, boolean isSpec) throws IOException {
		if (!fileOrFolder.exists())
			return;
		if (fileOrFolder.isDirectory())
			findIgnorableTables(new File(fileOrFolder, "dbsanity.xml"), false);
		if (fileOrFolder.isFile() && (isSpec || fileOrFolder.getName().equals("dbsanity.xml"))) {
			Document doc = XMLUtil.parse(fileOrFolder.getAbsolutePath());
			String ignorableTables = doc.getDocumentElement().getAttribute("ignorableTables");
			this.ignorableTables = ignorableTables;
		}
	}

	// private helpers ---------------------------------------------------------

	private Database getMetaData() throws ImportFailedException, ConnectFailedException, IOException, SQLException {
		DBMetaDataImporter importer = createImporter();
		if (metaDataCached)
			importer = new CachingDBImporter(importer, environment);
		return importer.importDatabase();
	}

	private KeyMapper createKeyMapper(Connection connection, String dbName, IdentityProvider identityProvider) {
		return new MemKeyMapper(connection, dbName, null, null, identityProvider);
	}

	private JDBCDBImporter createImporter() throws ConnectFailedException, ImportFailedException {
		JDBCDBImporter importer = new JDBCDBImporter(
				connectData.url, connectData.driver, connectData.user, connectData.password, 
				connectData.catalog, connectData.schema);
		importer.setExcludeTables(ignorableTables);
		importer.setLazy(true);
		importer.setImportingUKs(false);
		importer.setImportingIndexes(false);
		return importer;
	}

	private ReportContext createContext() {
		ReportContext context = new ReportContext(reportFolder, appVersion, tables, locale);
		context.set("appVersion", appVersion);
		context.set("systemUser", SystemInfo.getUserName());
		context.set("connectData", connectData);
		context.setIgnorableTables(ignorableTables);
		context.setTablesToCheck(tables);
		context.setTags(tags);
		context.setDefectCountLimit(defectCountLimit);
	    return context;
    }

	private Reporter createReporter(ReportContext context, Database database) throws IOException {
	    Reporter reporter = new Reporter(reportFolder, style, context);
	    String className = "org.databene.dbsanity.report." + StringUtil.capitalize(style) + "VerdictSummaryModule";
	    VerdictSummaryModule summaryModule = (VerdictSummaryModule) BeanUtil.newInstance(className);
	    
	    reporter.addModule(summaryModule);
		reporter.addModule(new InfoModule());

	    reporter.addModule(new DigestModule());
	    reporter.addModule(new CheckAssessmentModule());

	    reporter.addModule(new DefectsPerTableModule());
	    reporter.addModule(new UncheckedTablesModule(database, rootSuite.getIgnorableTables()));

	    reporter.addModule(new DefectsPerTypeModule());

	    reporter.addModule(new DefectsPerCheckModule());
	    
	    reporter.addModule(new SubSuiteModule(condensed));
	    
	    reporter.addModule(new ExecutionTimeModule());

	    reporter.addModule(new DefectsPageModule());
	    reporter.addModule(new ErrorModule());
	    reporter.addModule(new TempResultFileModule());
	    reporter.setContext(context);
	    return reporter;
    }

	private void checkPreconditions() {
	    if (checkDefinitionFile == null)
			throw new ConfigurationError(getClass().getSimpleName()
					+ ".testDefinitionFolder is null");
		if (!checkDefinitionFile.exists())
			throw new ConfigurationError("Test definition folder not found: "
					+ checkDefinitionFile.getAbsolutePath());
		if (connectData == null)
			throw new ConfigurationError(getClass().getSimpleName()
					+ ".connectData is null");
    }

	private Connection openConnection() throws ConnectFailedException {
		return DBUtil.connect(connectData.url, connectData.driver,
				connectData.user, connectData.password, true);
	}

	public void setDefectCountLimit(int defectCountLimit) {
		this.defectCountLimit = defectCountLimit;
	}

	public void setTags(Set<String> tags) {
		this.tags = tags;
	}

}
