package org.databene.dbsanity.model;

import java.io.File;
import java.sql.Connection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.databene.commons.collection.OrderedNameMap;
import org.databene.dbsanity.Reporter;
import org.databene.dbsanity.report.ReportContext;

/**
 * Groups files and sub folders to a composite sanity check.
 * @since 1.0
 * @author Yibo Wang
 * @author Volker Bergmann
 */
public class SanityCheckFolder extends SanityCheckSuite {
	
	protected Map<String, SanityCheckSuite> childSuites;
	private Date startDate;

	public SanityCheckFolder(String name, File reportFolder, SanityCheckFolder parent) {
		super(name, reportFolder, parent);
		this.childSuites = new OrderedNameMap<SanityCheckSuite>();
	}
	
	public void addChildSuite(SanityCheckSuite component) {
		this.childSuites.put(component.getName(), component);
	}
	
	public Map<String, SanityCheckSuite> getChildSuites() {
	    return childSuites;
    }

	@SuppressWarnings("unchecked")
    public <T extends SanityCheckSuite> Map<String, T> getChildSuites(Class<T> suiteType) {
		Map<String, T> result = new HashMap<String, T>();
		for (Entry<String, SanityCheckSuite> entry : childSuites.entrySet())
			if (suiteType.isAssignableFrom((entry.getValue().getClass())))
				result.put(entry.getKey(), (T) entry.getValue());
	    return result;
    }

	@Override
	public int getCheckCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getCheckCount();
		return count;
	}

	@Override
    public void perform(Connection connection, Reporter report, ReportContext context) {
		startDate = new Date();
		Iterator<SanityCheckSuite> iterator = childSuites.values().iterator();
		
		// perform folders first - they are assumed to contain sub modules  
		while (iterator.hasNext()) {
			SanityCheckSuite suite = iterator.next();
			if (suite instanceof SanityCheckFolder)
				suite.perform(connection, report, context);
		}
		
		// finally perform all other suites - they are assumed to perform integration chacks over sub modules
		iterator = childSuites.values().iterator();
		while (iterator.hasNext()) {
			SanityCheckSuite suite = iterator.next();
			if (!(suite instanceof SanityCheckFolder))
				suite.perform(connection, report, context);
		}
	}
	
	public Date getStartDate() {
		return startDate;
	}
	
	public boolean containsSubFolders() {
		for (SanityCheckSuite childSuite : childSuites.values())
			if (childSuite instanceof SanityCheckFolder)
				return true;
		return false;
	}

	public boolean containsCheckFiles() {
		for (SanityCheckSuite childSuite : childSuites.values())
			if (childSuite instanceof SanityCheckFile)
				return true;
		return false;
	}

	@Override
    public int getSkipCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getSkipCount();
		return count;
	}
	
	@Override
    public int getSuccessCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getSuccessCount();
		return count;
	}
	
	@Override
    public int getFailureCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getFailureCount();
		return count;
	}
	
	@Override
    public int getErrorCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getErrorCount();
		return count;
	}

	@Override
    public int getDefectCount() {
		int count = 0;
		for (SanityCheckSuite childSuite : childSuites.values())
			count += childSuite.getDefectCount();
		return count;
	}

	@Override
    public int getExecutionTime() {
		int sum = 0; 
		for (SanityCheckSuite childSuite : getChildSuites().values())
			sum += childSuite.getExecutionTime();
	    return sum;
    }

}
