/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.io.File;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

import org.databene.commons.ArrayUtil;
import org.databene.commons.Encodings;
import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.jdbacl.identity.IdentityProvider;
import org.databene.jdbacl.identity.KeyMapper;
import org.databene.jdbacl.model.Database;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

/**
 * Parses SQLUnit file hierarchies and provides them as a tree of {@link SanityCheckFolder}, 
 * {@link SanityCheckFile} and {@link SanityCheck} instances.<br/><br/>
 * Created: 13.10.2010 13:46:12
 * @since 1.0
 * @author Volker Bergmann
 */
public class TestSuiteParser {
	
	File reportRoot;
	
	// interface ---------------------------------------------------------------

	public SanityCheckSuite parseHierarchy(File definitionFileOrFolder, File reportRoot,
			File reportFolder, File tempFolder, Database database, KeyMapper keyMapper, IdentityProvider identityProvider) throws IOException {
		this.reportRoot = reportRoot;
        DbSanityParseContext context = new DbSanityParseContext(reportFolder, tempFolder, identityProvider, database, keyMapper);
		return parseSuite(definitionFileOrFolder, context, new Object[0]);
    }
	
	// private helper methods --------------------------------------------------

	private SanityCheckSuite parseSuite(File definitionFileOrFolder, DbSanityParseContext context, Object[] parentPath) throws IOException {
		if (!definitionFileOrFolder.exists())
			throw new RuntimeException("Test definition not found: " + definitionFileOrFolder);
		SanityCheckSuite result;
    	SanityCheckFolder parent = ParseUtil.getParentSanityCheckFolder(parentPath);
	    if (definitionFileOrFolder.isDirectory()) {
	    	if (parent == null)
    			result = parseFolder(definitionFileOrFolder, context, parentPath);
	    	else {
	    		File tempFolder = FileUtil.fileOfLimitedPathLength(context.getTempFolder(), definitionFileOrFolder.getName(), ".result");
	    		File reportFolder = new File(context.getReportFolder(), definitionFileOrFolder.getName());
	    		DbSanityParseContext subContext = context.createSubContext(definitionFileOrFolder, reportFolder, tempFolder);
	    		result = parseFolder(definitionFileOrFolder, subContext, parentPath);
	    	}
	    } else if (definitionFileOrFolder.isFile()) {
	    	if (parent == null)
    			result = parseFile(definitionFileOrFolder, context, parentPath);
	    	else {
		    	String subFolderName = cutSuffix(definitionFileOrFolder.getName());
		    	File subReportFolder = new File(context.getReportFolder(), subFolderName);
		    	File subTempFolder = new File(context.getTempFolder(), subFolderName);
		    	DbSanityParseContext subContext = context.createSubContext(definitionFileOrFolder, subReportFolder, subTempFolder);
				result = parseFile(definitionFileOrFolder, subContext, parentPath);
	    	}
	    } else
			throw new RuntimeException(
					"Test definition folder is neither a Directory nor a File: " + definitionFileOrFolder);
		// create TestSuiteHierachie including TestStuiteSanityCheck(s)
	    return result;
    }

	static String cutSuffix(String fileName) {
		return fileName.substring(0, fileName.lastIndexOf('.'));
    }

	private SanityCheckFolder parseFolder(File definitionFolder, DbSanityParseContext context, Object[] parentPath) throws IOException {
		String name = definitionFolder.getName();
		SanityCheckFolder result = new SanityCheckFolder(name, context.getReportFolder(), ParseUtil.getParentSanityCheckFolder(parentPath));
		for (File subFile : listSuites(definitionFolder)) {
			Object[] subPath = ArrayUtil.append(result, parentPath);
			result.addChildSuite(parseSuite(subFile, context, subPath));
		}
	    return result;
    }

	private List<File> listSuites(File definitionFolder) {
		List<File> result = new ArrayList<File>();
	    for (File file : definitionFolder.listFiles())
	    	if (file.isDirectory() || file.getName().toLowerCase().endsWith(".xml"))
	    		result.add(file);
	    return result;
    }

	private SanityCheckFile parseFile(File definition, DbSanityParseContext context, Object[] parentPath) 
			throws IOException {
        String xmlString = IOUtil.getContentOfURI(definition.getCanonicalPath());
        Document doc = XMLUtil.parseString(xmlString, new SqlUnitEntityResolver(definition.getParent()));
        DbSanityParseContext subContext = context.createSubContext(definition, context.getReportFolder(), context.getTempFolder());
		return (SanityCheckFile) subContext.parseElement(doc.getDocumentElement(), parentPath);
    }

	// helper class ------------------------------------------------------------

	public static class SqlUnitEntityResolver implements EntityResolver {
    	
    	private String contextUri;
    	
	    public SqlUnitEntityResolver(String contextUri) {
	        this.contextUri = contextUri;
        }

		public InputSource resolveEntity(String publicId, String systemId) throws IOException {
			systemId = URLDecoder.decode(systemId, Encodings.UTF_8);
	    	String absoluteUri = IOUtil.resolveRelativeUri(systemId, contextUri);
			return new InputSource(IOUtil.getInputStreamForURI(absoluteUri));
	    }

    }

}
