/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import static org.databene.dbsanity.report.ReportUtil.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.databene.commons.FileUtil;
import org.databene.commons.NameComparator;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;

/**
 * Provides an overview of a DB sanity sub suite.<br/><br/>
 * Created: 08.11.2010 17:51:11
 * @since 0.4
 * @author Volker Bergmann
 * @author Yibo Wang
 */
public class SubSuiteModule extends AbstractReportModule {
	
	private boolean condensed;
	
	public SubSuiteModule(boolean condensed) {
		this.condensed = condensed;
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return 2;
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite,
			FilePrintStream out) {
		try {
			FileUtil.ensureDirectoryExists(suite.getReportFolder());
			ReportUtil.copyResourceFiles(suite.getReportFolder(), style);
			if (suite instanceof SanityCheckFolder)
				createFolderSummary((SanityCheckFolder) suite, true, out);
			else
				createFileSummary((SanityCheckFile) suite, true, out);
		} catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void close() {
		super.close();
	}
	
	// private helper methods --------------------------------------------------
	
	private void createFolderSummary(SanityCheckFolder folder, boolean isRoot, FilePrintStream out) {
        groupDiv(folder, out);
        filesDiv(folder, out);
	}

	private void createFileSummary(SanityCheckFile file, boolean isRoot, FilePrintStream out) {
        checkDiv(file, out);
	}

	private void groupDiv(SanityCheckFolder parent, FilePrintStream out) {
		if (parent.containsSubFolders()) {
			startModule(out);
			suitesTable("Test Groups", parent, SanityCheckFolder.class, out);
			endModule(out);
		}
	}

	private void filesDiv(SanityCheckFolder parent, FilePrintStream out) {
		if (parent.containsCheckFiles()) {
			startModule(out);
			suitesTable("Test Files", parent, SanityCheckFile.class, out);
            checkTables(parent, out);
			endModule(out);
		}
	}

	private void suitesTable(String title, SanityCheckFolder parent, Class<? extends SanityCheckSuite> suiteType, FilePrintStream out) {
		modHeader(title, out);
		startModBody(out);
	    out.println("					<table width='100%' cellpadding='3' border='0'>");
	    out.println("						<thead>");
	    out.println("							<tr>" +
	    			"								<th width='5%'  align='left'>Verdict</th>" +
	    			"								<th width='20%' align='left'>" + (suiteType == SanityCheckFile.class ? "File" : "Group") + "</th>" +
	    			"								<th width='5%'  align='right'>Defects</th>" +
	    			"								<th width='70%' align='left'>Verdicts</th>"+ 
	    			"							</tr>");
	    out.println("						</thead>");
	    out.println("						<tbody>");
	    List<SanityCheckSuite> childSuites = new ArrayList<SanityCheckSuite>(parent.getChildSuites(suiteType).values());
	    Collections.sort(childSuites, new NameComparator());
		int maxTestCount = 0;
		for (SanityCheckSuite childSuite : childSuites)
            maxTestCount = Math.max(getTestCount(childSuite), maxTestCount);
	    for (SanityCheckSuite child : childSuites) {
    		out.println("							<tr>");
    		out.println("								<td align='center'>" + context.verdictImage(child.getVerdict(), out.getFile()) + "</td>");
			out.println("								<td align='left'>" + fileOrFolderRef(child) + "</td>");
			out.println("								<td align='right' class='defect'>" + (child.getDefectCount()  > 0 ? context.formatLong(child.getDefectCount())  : "&nbsp;") + "</td>");
    		out.println("								<td width='75%' align='left'>");
    		summaryBar(child, (100 * getTestCount(child) / (double) maxTestCount) + "%", out);
    		out.println("								</td>");
    		out.println("							</tr>");
	    }
	    out.println("						</tbody>");
	    out.println("					</table>");
	    endModBody(out);
    }

	private String fileOrFolderRef(SanityCheckSuite child) {
		String result = "";
		if (child instanceof SanityCheckFolder) {
			if (!((SanityCheckFolder) child).getChildSuites().isEmpty())
				result = "<a href=\"" + child.getName() + "/index.html\">";
		} else
			result = "<a href=\"#" + child.getName() + "\">";
		result += child.getName();
		return result;
	}

	private void checkTables(SanityCheckFolder folder, FilePrintStream out) {
		if (folder.containsCheckFiles()) {
		    List<SanityCheckSuite> childSuites = new ArrayList<SanityCheckSuite>(folder.getChildSuites(SanityCheckFile.class).values());
			Collections.sort(childSuites, new NameComparator());
			for (SanityCheckSuite child : childSuites)
				checkDiv(child, out);
		}
	}

	private void checkDiv(SanityCheckSuite child, FilePrintStream out) {
	    startModule(out);
	    out.println("	<a name=\"" + child.getName() + "\"/>");
	    modHeader(child.getName(), out);
	    startModBody(out);
	    checkTable(child, out);
	    endModBody(out);
	    endModule(out);
    }

	private void checkTable(SanityCheckSuite child, FilePrintStream out) {
	    out.println("	<table width='100%' cellpadding='3' border='0' cellspacing='0'>");
	    out.println("		<tr>" +
	    			"			<th width='5%'  class='trow' align='center'>Verdict</th>" +
	    			"			<th width='75%' class='trow' align='left'>Name</th>" +
	    			"			<th width='10%' class='trow' align='right'>Defects</th>" +
	    			"			<th width='10%' class='trow' align='right'>Duration</th>" +
					"		</tr>");
	    for (SanityCheck check : ((SanityCheckFile) child).getChecks().values()) {
	    	SanityCheckVerdict verdict = check.getVerdict();
	    	if (!condensed || verdict == SanityCheckVerdict.FAILED || verdict == SanityCheckVerdict.ERROR) {
		    	out.print("		<tr>");
	    		out.println("			<td align='center' class='trow'>" + context.verdictImage(verdict, out.getFile()) + "</td>");
	    		String note = (check.getExecutionNote() != null ? " (" + check.getExecutionNote() + ')' : "");
				out.println("			<td align='left' class='trow'><a href='" + docOrDefectFileRef(check, out.getFile()) + "'>" + check.getName() + note + "</a></td>");
				out.println("			<td align='right' class='trow_defect'>" 
					+ context.formatLong(check.getDefectCount(), "&nbsp;") + "</td>");
				int time = check.getExecutionTime();
				out.println("			<td align='right' class='trow'>"
					+ (time > 0 ? context.formatLong(time) + " ms" : "") + "</td>");
		    	out.println("		</tr>");
	    	}
	    }
	    out.println("	</table>");
    }

}
